#include "Adafruit_Keypad.h"
int state = 1;
char buttonState;

const byte ROWS = 4; // rows
const byte COLS = 4; // columns
//define the symbols on the buttons of the keypads
char keys[ROWS][COLS] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};
byte rowPins[ROWS] = {4, 5, 6, 7}; //connect to the row pinouts of the keypad
byte colPins[COLS] = {8, 10, 12, 11}; //connect to the column pinouts of the keypad

//initialize an instance of class NewKeypad
Adafruit_Keypad customKeypad = Adafruit_Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS);

/*//////
  //FFT//
  /////*/
#define ARM_MATH_CM4
#include <arm_math.h>
#include <Adafruit_NeoPixel.h>


int SAMPLE_RATE_HZ = 6000 ;             // Sample rate of the audio in hertz.
float SPECTRUM_MIN_DB = 45.0;          // Audio intensity (in decibels) that maps to low LED brightness.
float SPECTRUM_MAX_DB = 90.0;          // Audio intensity (in decibels) that maps to high LED brightness.
int LEDS_ENABLED = 1;                  // Control if the LED's should display the spectrum or not.  1 is true, 0 is false.
// Useful for turning the LED display on and off with commands from the serial port.
const int FFT_SIZE = 256;              // Size of the FFT.  Realistically can only be at most 256
// without running out of memory for buffers and other state.
const int AUDIO_INPUT_PIN = 14;        // Input ADC pin for audio data.
const int ANALOG_READ_RESOLUTION = 10; // Bits of resolution for the ADC.
const int ANALOG_READ_AVERAGING = 16;  // Number of samples to average with each ADC reading.
const int POWER_LED_PIN = 13;          // Output pin for power LED (pin 13 to use Teensy 3.0's onboard LED).
const int NEO_PIXEL_PIN = 3;           // Output pin for neo pixels.
const int NEO_PIXEL_COUNT = 300;        // Number of neo pixels.  You should be able to increase this without
// any other changes to the program.
const int MAX_CHARS = 65;              // Max size of the input command buffer


IntervalTimer samplingTimer;
float samples[FFT_SIZE * 2];
float magnitudes[FFT_SIZE];
int sampleCounter = 0;
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NEO_PIXEL_COUNT, NEO_PIXEL_PIN, NEO_GRB + NEO_KHZ800);
char commandBuffer[MAX_CHARS];
float frequencyWindow[NEO_PIXEL_COUNT + 1];
float hues[NEO_PIXEL_COUNT];


//POTENTIOMETER CONTROLS
#define NEO_PIXEL_PIN 3 //input pin Neopixel is attached to

#define NEO_PIXEL_COUNT 300 //number of neopixels in strip;

int delayval = 1; //time delay in milliseconds

int redColor; //initialize variables
int greenColor;
int blueColor;


//FADE
#include <FastLED.h>
#define DATA_PIN 3
#define NUM_LEDS 300
#define LED_COUNT 300
#define COLOR_ORDER GRB

CRGB leds[NUM_LEDS];

//RNG
int redColor1 = 0;
int greenColor1 = 0;
int blueColor1 = 0;


void setup() {
  Serial.begin(38400);
  customKeypad.begin();

  pinMode(AUDIO_INPUT_PIN, INPUT);
  analogReadResolution(ANALOG_READ_RESOLUTION);
  analogReadAveraging(ANALOG_READ_AVERAGING);

  // Turn on the power indicator LED.
  pinMode(POWER_LED_PIN, OUTPUT);
  digitalWrite(POWER_LED_PIN, HIGH);

  // Initialize neo pixel library and turn off the LEDs
  pixels.begin();
  pixels.show();

  // Clear the input command buffer
  memset(commandBuffer, 0, sizeof(commandBuffer));

  // Initialize spectrum display
  spectrumSetup();

  // Begin sampling audio
  samplingBegin();

  Serial.begin(9600); //initialize serial
  pixels.begin(); //Initialize the NeoPixel library

  FastLED.addLeds<WS2811, DATA_PIN, GRB>(leds, NUM_LEDS);
}

void loop() {
  // put your main code here, to run repeatedly:
  customKeypad.tick();

  while (customKeypad.available()) {
    keypadEvent e = customKeypad.read();
    // Serial.print((char)e.bit.KEY);
    buttonState = (char)e.bit.KEY;
    if (e.bit.EVENT == KEY_JUST_PRESSED) {
      //Serial.print(buttonState);
      Serial.println(" pressed");
    }
    //else if(e.bit.EVENT == KEY_JUST_RELEASED) Serial.println(" released");
  }



  switch (state) {
    case 1:
      Serial.println(" In state 1");
      FFT();
      if (buttonState == '2') {
        state = 2;
        break;
      }
      else if (buttonState == '3') {
        state = 3;
        break;
      }
      else if (buttonState == '4') {
        state = 4;
        break;
      }
      else if (buttonState == '5') {
        state = 5;
        break;
      }
      else if (buttonState == '6') {
        state = 6;
        break;
      }

      //checkState();
      break;


    case 2:
      Serial.println(" In state 2");
     PotControl();
      checkState();

      break;


    case 3:
      Serial.println(" In state 3");
      //FFTBASS();
      checkState();

      break;


    case 4:
      Serial.println(" In state 4");
      // FFTTREBLE();
      checkState();
      break;


    case 5:
      Serial.println(" In state 5");
       FADE();
      checkState();
      break;


    case 6:
      Serial.println(" In state 6");
       RNG();
      checkState();
      break;
  }

}

void checkState() {
  if (buttonState == '1') {
    state = 1;
   
  }
   else if (buttonState == '2') {
    state = 2;
    
  }
  else if (buttonState == '3') {
    state = 3;
 
  }
  else if (buttonState == '4') {
    state = 4;
    
  }
  else if (buttonState == '5') {
    state = 5;
   
  }
  else if (buttonState == '6') {
    state = 6;
   
  }
}
void FFT() {
  // Calculate FFT if a full sample is available.
  if (samplingIsDone()) {
    // Run FFT on sample data.
    arm_cfft_radix4_instance_f32 fft_inst;
    arm_cfft_radix4_init_f32(&fft_inst, FFT_SIZE, 0, 1);
    arm_cfft_radix4_f32(&fft_inst, samples);
    // Calculate magnitude of complex numbers output by the FFT.
    arm_cmplx_mag_f32(samples, magnitudes, FFT_SIZE);

    if (LEDS_ENABLED == 1)
    {
      spectrumLoop();
    }

    // Restart audio sampling.
    samplingBegin();
  }

  // Parse any pending commands.
  parserLoop();
}

// Compute the average magnitude of a target frequency window vs. all other frequencies.
void windowMean(float * magnitudes, int lowBin, int highBin, float * windowMean, float * otherMean) {
  *windowMean = 0;
  *otherMean = 0;
  // Notice the first magnitude bin is skipped because it represents the
  // average power of the signal.
  for (int i = 1; i < FFT_SIZE / 2; ++i) {
    if (i >= lowBin && i <= highBin) {
      *windowMean += magnitudes[i];
    }
    else {
      *otherMean += magnitudes[i];
    }
  }
  *windowMean /= (highBin - lowBin) + 1;
  *otherMean /= (FFT_SIZE / 2 - (highBin - lowBin));
}

// Convert a frequency to the appropriate FFT bin it will fall within.
int frequencyToBin(float frequency) {
  float binFrequency = float(SAMPLE_RATE_HZ) / float(FFT_SIZE);
  return int(frequency / binFrequency);
}

// Convert from HSV values (in floating point 0 to 1.0) to RGB colors usable
// by neo pixel functions.
uint32_t pixelHSVtoRGBColor(float hue, float saturation, float value) {
  // Implemented from algorithm at http://en.wikipedia.org/wiki/HSL_and_HSV#From_HSV
  float chroma = value * saturation;
  float h1 = float(hue) / 60.0;
  float x = chroma * (1.0 - fabs(fmod(h1, 2.0) - 1.0));
  float r = 0;
  float g = 0;
  float b = 0;
  if (h1 < 1.0) {
    r = chroma;
    g = x;
  }
  else if (h1 < 2.0) {
    r = x;
    g = chroma;
  }
  else if (h1 < 3.0) {
    g = chroma;
    b = x;
  }
  else if (h1 < 4.0) {
    g = x;
    b = chroma;
  }
  else if (h1 < 5.0) {
    r = x;
    b = chroma;
  }
  else // h1 <= 6.0
  {
    r = chroma;
    b = x;
  }
  float m = value - chroma;
  r += m;
  g += m;
  b += m;
  return pixels.Color(int(255 * r), int(255 * g), int(255 * b));
}

void spectrumSetup() {
  // Set the frequency window values by evenly dividing the possible frequency
  // spectrum across the number of neo pixels.
  float windowSize = (SAMPLE_RATE_HZ / 2.0) / float(NEO_PIXEL_COUNT);
  for (int i = 0; i < NEO_PIXEL_COUNT + 1; ++i) {
    frequencyWindow[i] = i * windowSize;
  }
  // Evenly spread hues across all pixels.
  for (int i = 0; i < NEO_PIXEL_COUNT; ++i) {
    hues[i] = 360.0 * (float(i) / float(NEO_PIXEL_COUNT - 1));
  }
}

void spectrumLoop() {
  // Update each LED based on the intensity of the audio
  // in the associated frequency window.
  float intensity, otherMean;
  for (int i = 0; i < NEO_PIXEL_COUNT; ++i) {
    windowMean(magnitudes,
               frequencyToBin(frequencyWindow[i]),
               frequencyToBin(frequencyWindow[i + 1]),
               &intensity,
               &otherMean);
    // Convert intensity to decibels.
    intensity = 20.0 * log10(intensity);
    // Scale the intensity and clamp between 0 and 1.0.
    intensity -= SPECTRUM_MIN_DB;
    intensity = intensity < 0.0 ? 0.0 : intensity;
    intensity /= (SPECTRUM_MAX_DB - SPECTRUM_MIN_DB);
    intensity = intensity > 1.0 ? 1.0 : intensity;
    pixels.setPixelColor(i, pixelHSVtoRGBColor(hues[i], 1.0, intensity));
  }
  pixels.show();
}

void samplingCallback() {
  // Read from the ADC and store the sample data
  samples[sampleCounter] = (float32_t)analogRead(AUDIO_INPUT_PIN);
  // Complex FFT functions require a coefficient for the imaginary part of the input.
  // Since we only have real data, set this coefficient to zero.
  samples[sampleCounter + 1] = 0.0;
  // Update sample buffer position and stop after the buffer is filled
  sampleCounter += 2;
  if (sampleCounter >= FFT_SIZE * 2) {
    samplingTimer.end();
  }
}

void samplingBegin() {
  // Reset sample buffer position and start callback at necessary rate.
  sampleCounter = 0;
  samplingTimer.begin(samplingCallback, 1000000 / SAMPLE_RATE_HZ);
}

boolean samplingIsDone() {
  return sampleCounter >= FFT_SIZE * 2;
}

void parserLoop() {
  // Process any incoming characters from the serial port
  while (Serial.available() > 0) {
    char c = Serial.read();
    // Add any characters that aren't the end of a command (semicolon) to the input buffer.
    if (c != ';') {
      c = toupper(c);
      strncat(commandBuffer, &c, 1);
    }
    else
    {
      // Parse the command because an end of command token was encountered.
      parseCommand(commandBuffer);
      // Clear the input buffer
      memset(commandBuffer, 0, sizeof(commandBuffer));
    }
  }
}

// Macro used in parseCommand function to simplify parsing get and set commands for a variable
#define GET_AND_SET(variableName) \
  else if (strcmp(command, "GET " #variableName) == 0) { \
    Serial.println(variableName); \
  } \
  else if (strstr(command, "SET " #variableName " ") != NULL) { \
    variableName = (typeof(variableName)) atof(command+(sizeof("SET " #variableName " ")-1)); \
  }

void parseCommand(char* command) {
  if (strcmp(command, "GET MAGNITUDES") == 0) {
    for (int i = 0; i < FFT_SIZE; ++i) {
      Serial.println(magnitudes[i]);
    }
  }
  else if (strcmp(command, "GET SAMPLES") == 0) {
    for (int i = 0; i < FFT_SIZE * 2; i += 2) {
      Serial.println(samples[i]);
    }
  }
  else if (strcmp(command, "GET FFT_SIZE") == 0) {
    Serial.println(FFT_SIZE);
  }
  GET_AND_SET(SAMPLE_RATE_HZ)
  GET_AND_SET(LEDS_ENABLED)
  GET_AND_SET(SPECTRUM_MIN_DB)
  GET_AND_SET(SPECTRUM_MAX_DB)

  // Update spectrum display values if sample rate was changed.
  if (strstr(command, "SET SAMPLE_RATE_HZ ") != NULL) {
    spectrumSetup();
  }

  // Turn off the LEDs if the state changed.
  if (LEDS_ENABLED == 0) {
    for (int i = 0; i < NEO_PIXEL_COUNT; ++i) {
      pixels.setPixelColor(i, 0);
    }
    pixels.show();
  }
}


void PotControl() {
  int redColor = analogRead(1) / 4.02; //Take the analog value of the potentiometer (0-1023)
  int greenColor = analogRead(2) / 4.02; //Divide it by 4.02 so that this value is between 0-255.
  int blueColor = analogRead(3) / 4.02; //the division for 4.02 prevents the value from exceeding 255

  /* //prints the serial values of each potentiometer (Red, Green, Blue).
    Serial.print("r: ");
    Serial.println(redColor);


    Serial.print("g: ");
    Serial.println(greenColor);


    Serial.print("b: ");
    Serial.println(blueColor);

    Serial.println("");
  */

  for (int i = 0; i < NEO_PIXEL_COUNT; i++) {
    //pixels.Color takes RGB values, from 0,0,0 up to 255,255,255
    pixels.setPixelColor(i, pixels.Color(redColor, greenColor, blueColor));

    //this sends the updated pixel color to the hardware.
    pixels.show();

    //delay for a period of time (in milliseconds).
    delay(delayval);
  }
}

void FFTBASS() {
}
void FFTTREBLE() {
}


void FADE() {

  //start from red
  for ( int colorStep = 0; colorStep <= 255; colorStep++ ) {

    int r = 255;
    int g = 0;
    int b = colorStep;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    FastLED.show();
  }

  //into blue
  for ( int colorStep = 255; colorStep >= 0; colorStep-- ) {

    int r = colorStep;
    int g = 0;
    int b = 255;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    FastLED.show();
  }

  //start from blue
  for ( int colorStep = 0; colorStep <= 255; colorStep++ ) {

    int r = 0;
    int g = colorStep;
    int b = 255;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    FastLED.show();
  }

  //into green
  for ( int colorStep = 255; colorStep >= 0; colorStep-- ) {

    int r = 0;
    int g = 255;
    int b = colorStep;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    LEDS.show();
  }

  //start from green
  for ( int colorStep = 0; colorStep <= 255; colorStep++ ) {

    int r = colorStep;
    int g = 255;
    int b = 0;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    LEDS.show();
  }

  //into yellow
  for ( int colorStep = 255; colorStep >= 0; colorStep-- ) {

    int r = 255;
    int g = colorStep;
    int b = 0;

    // Now loop though each of the LEDs and set each one to the current color
    for (int x = 0; x < LED_COUNT; x++) {
      leds[x] = CRGB(r, g, b);
    }

    // Display the colors we just set on the actual LEDs
    delay(10);
    LEDS.show();
  }
}



void RNG() {
  setColor();

  for (int i = 0; i < NEO_PIXEL_COUNT; i++) {

    // pixels.Color takes RGB values, from 0,0,0 up to 255,255,255
    pixels.setPixelColor(i, pixels.Color(redColor1, greenColor1, blueColor1)); // Moderately bright green color.

    pixels.show(); // This sends the updated pixel color to the hardware.

    delay(delayval); // Delay for a period of time (in milliseconds).

    // Serial.println(i);

    if (i == NEO_PIXEL_COUNT) {
      i = 0; // start all over again!
      setColor();
    }

  }
}


// setColor()
// picks random values to set for RGB

void setColor() {
  redColor1 = random(0, 255);
  greenColor1 = random(0, 255);
  blueColor1 = random(0, 255);
  Serial.print("red: ");
  Serial.println(redColor);
  Serial.print("green: ");
  Serial.println(greenColor);
  Serial.print("blue: ");
  Serial.println(blueColor);


}
